package leigh.dnsd;

import leigh.mecha.log.MechaLogger;
import leigh.mecha.log.MechaLoggerFactory;
import org.xbill.DNS.Record;
import org.xbill.DNS.*;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.UnknownHostException;

public class DNSService implements Runnable {
    private static final MechaLogger logger = MechaLoggerFactory.getLogger(DNSService.class);
    private static final int UDP_SIZE = 512;
    private final int port;
    private final SimpleResolver resolver;

    DNSService(int port) throws UnknownHostException {
        this.port = port;
        resolver = new SimpleResolver("8.8.8.8");
    }

    public void run() {
        try {
            DatagramSocket socket = new DatagramSocket(port);
            //noinspection InfiniteLoopStatement
            while (true)
                process(socket);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void process(DatagramSocket socket) throws IOException {
        byte[] in = new byte[UDP_SIZE];
        DatagramPacket indp = new DatagramPacket(in, UDP_SIZE);
        socket.receive(indp);
        Message request = new Message(in);

        Name name = request.getQuestion().getName();

        Message reply;

        if (name.toString().endsWith("leigh-co.com.")) {
            logger.info("local query");
            reply = new Message(request.getHeader().getID());
            reply.addRecord(request.getQuestion(), Section.QUESTION);
            reply.addRecord(Record.fromString(Name.root, Type.A, DClass.IN, 86400, "1.2.3.4", Name.root),
                    Section.ANSWER);
        } else {
            logger.info("recursed query");
            reply = resolver.send(request);
        }

        byte[] resp = reply.toWire();
        DatagramPacket outdp = new DatagramPacket(resp, resp.length,
                indp.getAddress(), indp.getPort());
        socket.send(outdp);
    }
}